# --8<--8<--8<--8<--
#
# Copyright (C) 2008 Smithsonian Astrophysical Observatory
#
# This file is part of App::Env::HEADAS
#
# App::Env::HEADAS is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -->8-->8-->8-->8--

package App::Env::HEADAS;

use strict;
use warnings;
use Carp;

our $VERSION = '0.08';

## no critic ( ProhibitAccessOfPrivateData )

use Params::Validate qw( validate_with );
use SysArch;
use Shell::GetEnv;
use File::Spec::Functions qw( catfile catdir);
use File::Basename qw( fileparse );

=pod

=begin making_pod_coverage_shut_up

=item envs

=end making_pod_coverage_shut_up

=cut


sub envs
{
    my ( $opt ) = @_; 

    my %opt = validate_with( params => \@_, 
                             spec => { version => 0,
                                       root => 0,
                                       platform => 0,
                                       verbose => 0,
                                     },
                             normalize_keys => sub { return lc $_[0] },
                           );

    my $setup_file = eval { _get_setup_file( $opt{root}, $opt{platform},
                                             $opt{version}, $opt{verbose} ) } ;

    die( "unable to locate HEADAS setup file\n" )
      unless defined $setup_file;

    warn( "Found $setup_file\n" ) if $opt{verbose};

    my $headas = ( fileparse( $setup_file) )[1];

    # source the shell script and return the changed environment
    # pass Shell::GetEnv's croaks up to caller of App::Env
    return Shell::GetEnv->new( 'tcsh',
                               "setenv HEADAS '$headas'",
                               "source $setup_file",
                             { Startup => 0 }
                             )->envs;
}

sub _get_setup_file
{
    my ( $root, $platform, $version, $verbose ) = @_;


    my @root;

    if ( defined $root )
    {
        @root = ( $root );
    }
    elsif ( defined $ENV{APP_ENV_HEADAS_ROOT} )
    {
        @root = ( $ENV{APP_ENV_HEADAS_ROOT} );
    }
    else
    {
        @root = qw( /proj/axaf/ots/pkgs /soft/lheasoft );
    }

    my $setup_file;

    foreach ( @root )
    {
        last if defined
          ( $setup_file = _get_setup_file_root( $_, $platform, $version, $verbose ) );
    }

    return $setup_file;
}


sub _get_setup_file_root
{

    my( $root, $platform, $version, $verbose ) = @_;

    if ( ! defined $version )
    {
        if ( exists  $ENV{APP_ENV_HEADAS_VERSION})
        {
            $version = '-' . $ENV{APP_ENV_HEADAS_VERSION};
        }
        else
        {
            $version = '';
        }
    }
    else
    {
        $version = '-' . $version;
    }


    my $headas = catdir( $root, 'headas' . $version );

    warn( "Checking $headas\n" ) if $verbose;

    return if ! -e $headas;

    my @platforms;

    $platform ||= $ENV{APP_ENV_HEADAS_PLATFORM};

    if ( defined $platform )
    {
        @platforms = ( $platform );
    }
    else
    {
        # now find platform specific directory
        my $os_type = $SysArch::Vars{os_type};

        if ( 'solaris' eq $os_type )
        {
            for my $release ( split(':', $SysArch::Vars{os_versions} ) )
            {
                $release -= 3;
                push @platforms, "sparc-sun-solaris$release";
            }
        }

        elsif ( 'linux' eq $os_type )
        {
            @platforms =
              (
               sprintf( '%s-%s_%s-linux-gnu-libc%s',
                        $SysArch::Vars{hw_cpu_generic},
                        $SysArch::Vars{os_vendor},
                        $SysArch::Vars{os_version},
                        $SysArch::Vars{os_uversion} ),

               sprintf( '%s-%s_%s-linux-gnu',
                        $SysArch::Vars{hw_cpu_generic},
                        $SysArch::Vars{os_vendor},
                        $SysArch::Vars{os_version} ),

               sprintf( '%s-unknown-linux-gnu',
                        $SysArch::Vars{hw_cpu_generic} ),

               sprintf( '%s-pc-linux-gnu-libc%s',
                        $SysArch::Vars{hw_cpu_generic},
                        $SysArch::Vars{os_uversion_maj} ),

               sprintf( '%s-pc-linux',
                        $SysArch::Vars{hw_cpu_generic} ),
              );


        }

        elsif ( 'darwin' eq $os_type )
        {
            my $hw   = `uname -m`;
            $hw = chomp $hw;
            $hw = 'i386' ? 'i686' : $hw;
            @platforms =
              (
               sprintf( '%s-%s-darwin%s',
                        $hw,
                        $SysArch::Vars{os_vendor},
                        $SysArch::Vars{kern_version} ),
              );
        }

        else
        {
            die( "Unsupported OS: $os_type\n" );
        }
    }

    warn( "Checking platforms: ", join( ', ', @platforms ), "\n" )
      if $verbose;

    my ( $dir ) = grep { -d catdir( $headas, $_ ) } @platforms;

    if ( defined $dir )
    {
        my $setup_file = catfile( $headas, $dir, 'headas-init.csh' );

        warn( "Checking $setup_file\n" ) if $verbose;

        return -e $setup_file ? $setup_file : undef;
    }

    return;
}

__END__

=head1 NAME

App::Env::HEADAS - Load the HEADAS (FTOOLS, XSPEC, etc) environment


=head1 SYNOPSIS

    use App::Env qw( HEADAS );

=head1 DESCRIPTION

This module should not be used directly; see B<App::Env>.  It uses the
F<headas-init.csh> script to generate the environment used to run
B<HEADAS> software.

Three pieces of information are used to determine the absolute path to
the F<headas-init.csh> script.

=over

=item 1

The top-level directory containing a version specific F<headas>
directory.  This may be provided either by the environment variable
B<APP_ENV_HEADAS_ROOT> or by specifying the C<root> option in
the B<App::Env::new> B<AppOpts> hash.  The latter has precedence over
the former.

=item 2

The version of B<HEADAS>.  This may be provided either by the
environment variable B<APP_ENV_HEADAS_VERSION> or by specifying the
C<version> option in the B<App::Env::new> B<AppOpts> hash.  The latter
has precedence over the former.

=item 3

The hardware/os platform.  This may be determined automatically (the
default) or may be explicitly provided either by the environment
variable B<APP_ENV_HEADAS_PLATFORM> or by specifying the C<platform>
option in the B<App::Env::new> B<AppOpts> hash.  The latter has
precedence over the former.  For Linux boxes, platform has the form

  ${hw_cpu_generic}-pc-linux

For Solaris boxes, platform has the form

  sparc-sun-solaris${os_version}

where the C<${}> variables are B<sysarch> variables of the same name.
(On Solaris it actually searches the list in B<os_versions>)

=back

The path to the F<headas-init.csh> script is either

   ROOT/headas/PLATFORM/headas-init.csh

if no version is provided, or

   ROOT/headas-VERSION/PLATFORM/headas-init.csh

if one is.

The following B<AppOpts> options are recognized:

=over

=item HEADAS_ROOT

The top level directory containing the F<headas> or F<headas-VERSION>
directory.

=item version

The version of HEADAS.  See above for how it is used.

=back

=head1 DEPENDENCIES

This module depends upon B<SysArch>, B<Shell::GetEnv>, B<File::Spec>.

=head1 BUGS AND LIMITATIONS

No bugs have been reported.

=head1 SEE ALSO

B<App::ENV>

=head1 VERSION

Version 0.06

=head1 LICENSE AND COPYRIGHT

Copyright (c) 2008 The Smithsonian Astrophysical Observatory

App::Env::HEADAS is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=head1 AUTHOR

Diab Jerius  E<lt>djerius@cpan.orgE<gt>


